import os, sys, ptMegaWidgets

class InterpBuffer:
    def __init__(self,filename,bufManager):
	self.filename = filename
	path,name = os.path.split(filename)

	if path: sys.path.insert(0,path)

	self.bufManager = bufManager
	self.name = bufManager.buildUniqueName(name)

	self.addToBuffersMenu()

	self.data = ''

	self.namespace = {}

	self._initNamespace()
     
    def _initNamespace(self):
	interp = self.bufManager.interp
	keys = interp.namespace.keys()
	for key in keys:
	    self.namespace[key] = interp.namespace[key]

	ric = ptMegaWidgets.RawInputCaller
	ic = ptMegaWidgets.InputCaller

	self.namespace['__builtins__']['raw_input'] = ric(interp)
	self.namespace['__builtins__']['input'] = ic(interp)


    def addToBuffersMenu(self):
	self.bufManager.interp.bufferMenu.menu.add_command(label=self.name,
							   command=self.switchTo)

    def removeFromBuffersMenu(self):
	self.bufManager.interp.bufferMenu.menu.delete(self.name)

    def updateFromScreen(self):
	self.data = self.bufManager.interp.iText.text.get('1.0','end')

    def updateFromFile(self):
	input = open(self.filename,'r')
	self.data = input.read()
	input.close()

    def switchTo(self,event=None):
	self.bufManager.switchToBuffer(self)

    def save(self):
	file = open(self.filename,'w')
	file.write(self.data)
	file.close()

    def saveAs(self,filename):
	self.filename = filename
	self.save()

	path,name = os.path.split(filename)
	if name != self.name:
	    name = self.bufManager.buildUniqueName(name)
	    self.bufManager.interp.bufferMenu.menu.entryconfig(self.name,
							       label=name)

	    del self.bufManager.buffers[self.name]
	    self.bufManager.buffers[name] = self

	    self.name = name

	self.switchTo()


class InterpBufferManager:
    ScratchName = 'Untitled'

    def __init__(self,interp):
	self.interp = interp

	self.current = None
	  
	self.buffers = {}
	  
	scratch = self.getScratch()
	self.switchToBuffer(scratch)

	self.last = self.current

    def addBuffer(self,filename):
	buf = self.newBuffer(filename)
	buf.updateFromFile()
	self.buffers[buf.name] = buf
	self.switchToBuffer(buf)

    def newBuffer(self,filename):
	return InterpBuffer(filename,self)
	  
    def deleteBuffer(self,buffer=None):
	if not buffer: buffer = self.current

	last = self.last
	self.current = None

	buffer.removeFromBuffersMenu()
	del self.buffers[buffer.name]

	path,name = os.path.split(buffer.filename)
	if path in sys.path:
	    sys.path.remove(path)

	scratch = self.getScratch()

	if last == buffer:
	    self.switchToBuffer(scratch)
	if last and (last != buffer):
	    self.switchToBuffer(last)
	else:
	    self.switchToBuffer(scratch)
	  
    def getScratch(self):
	if self.buffers.has_key(InterpBufferManager.ScratchName):
	    return self.buffers[InterpBufferManager.ScratchName]
	else:
	    scratch = InterpBuffer(InterpBufferManager.ScratchName,self)
	    self.buffers[scratch.name] = scratch

	    return scratch

    def saveBuffer(self):
	if not self.current: return

	self.current.updateFromScreen()
	self.current.save()

    def saveBufferAs(self,filename):
	if not self.current: return

	self.current.updateFromScreen()
	self.current.saveAs(filename)

	self.getScratch()                # ensure that "Untitled" exists

    def switchToBuffer(self,buffer):
	if self.current:
	    self.current.updateFromScreen()

	self.interp.iText.text.delete('1.0', 'end')

	self.interp.iText.text.insert('1.0', buffer.data)
	self.interp.iText.text.mark_set('insert','1.0')

	self.interp.iText.clearHistory()

	self.last = self.current
	self.current = buffer

	self.interp.setFilename(buffer.name)

    def buildUniqueName(self,name):
	uname = name
	post = 1
	while self.buffers.has_key(uname):
	    uname = '%s~%d' % (name,post)
	    post = post + 1

	return uname


