import os, string, Tkinter, FontText, ptMegaWidgets

class Text(ptMegaWidgets.ScrolledText):

    __undo_loaded = 0

    def __init__(self,root,text=None,readonly=None,indent_level=4,tab_size=8):
    	ptMegaWidgets.ScrolledText.__init__(self,root,text,readonly)
    	
    	self.indentLevel = indent_level
    	self.tabSize = tab_size

	if not Text.__undo_loaded:
	    path = os.path.split(__file__)[0]

	    self.tk.call('source',os.path.join(path,'new.tcl'))
	    self.tk.call('source',os.path.join(path,'lifo.tcl'))
	    self.tk.call('source',os.path.join(path,'textundo.tcl'))
	    
	    Text.__undo_loaded = 1

	self.undoerID = self.tk.call('new','textUndoer',self.text._w)
    
    def _initTkWidgets(self):
	self.scroll = Tkinter.Scrollbar(self)
	self.scroll.pack(side=Tkinter.RIGHT,
			 fill=Tkinter.Y)
	
	self.text = FontText.Text(self,
				  wrap=Tkinter.WORD,
				  yscrollcommand=self.scroll.set)
	
	self.text.pack(side=Tkinter.RIGHT,
		       expand=Tkinter.Y,
		       fill=Tkinter.BOTH)
	
	self.scroll.config(command=self.text.yview)
	
	self._initBindings()

    def _initBindings(self):
	self.text.bind('<Return>', self.break_then_indent)
	self.text.bind('<Tab>', self.indent)

    def destroy(self):
        self.tk.call('delete', 'textUndoer', self.undoerID)
        ptMegaWidgets.ScrolledText.destroy(self)

    def break_then_indent(self,event=None):
	self.text.insert('insert','\n')
	self.indent(event)
	return 'break'

    def indent(self,event=None):
	prev_pos = self._getPreviousNonEmpty('insert')

	indent = self._countIndent(prev_pos)

	last_char = self.text.get(prev_pos + ' lineend - 1 chars')
	
	if last_char in [':','\\']:
	    indent = indent + self.indentLevel

	self._removeIndentation()

	self.text.insert('insert linestart', self._buildIndentation(indent))

	return 'break'

    def _buildIndentation(self,level):
    	tabs,spaces = divmod(level,self.tabSize)
    	return '\t' * tabs + ' ' * spaces


    def _getPreviousNonEmpty(self,pos):
	pos = self.text.index(pos + ' - 1 lines')

	while self.text.index(pos + ' linestart') == self.text.index(pos + ' lineend'):
	    if pos == '1.0': break
	    pos = self.text.index(pos + ' - 1 lines')

	return pos


    def _countIndent(self,pos):
	count = 0
	for char in self.text.get(pos + ' linestart', pos + ' lineend'):
	    if char == '\t':
		count = count + self.tabSize

	    elif char == ' ':
	    	count = count + 1

	    else: break

	return count

    def _removeIndentation(self,pos='insert'):
    	indent_chars = [' ','\t']
	while self.text.get('%s linestart' % pos) in indent_chars:
	    self.text.delete('%s linestart' % pos)

    def getRegionLimits(self, selection):
    	begin = selection[0]
    	end = selection[-1]

	start = string.atoi(string.splitfields(begin, '.')[0])
	[finish, term] = string.splitfields(end, '.')
	finish = string.atoi(finish)
	if term == "0":
	    finish = finish - 1
	
	return start, finish

    def getSelectionLines(self,selection):
	start,finish = self.getRegionLimits(selection)   
	return map(lambda n: '%d.0' % n, range(start,finish + 1))
    	
    def commentRegion(self,event=None):
	sel = self.text.tag_ranges('sel')
	if sel:
	    for index in self.getSelectionLines(sel):
	    	self.text.insert(index,'#')
	    	
	    return 1

    def uncommentRegion(self,event=None):
	sel = self.text.tag_ranges('sel')
	if sel:
	    for index in self.getSelectionLines(sel):
	    	if self.text.get(index) == '#':
	    	    self.text.delete(index)
	    	
	    return 1

    def indentRegion(self,event=None):
	sel = self.text.tag_ranges('sel')
	if sel:
	    for index in self.getSelectionLines(sel):
	    	level = self._countIndent(index) + self.indentLevel
	    	self._removeIndentation(index)
	    	self.text.insert(index,self._buildIndentation(level))
	    	
	    return 1

    def dedentRegion(self,event=None):
	sel = self.text.tag_ranges('sel')
	if sel:
	    for index in self.getSelectionLines(sel):
		level = self._countIndent(index) - self.indentLevel
	    	self._removeIndentation(index)	    	
	    	
	    	if level >= 0:    
		    self.text.insert(index,self._buildIndentation(level))
	    	
	    return 1

    def undo(self, *event):
	return self.tk.call('textUndoer::undo',self.undoerID)

    def clearHistory(self, *event):
        return self.tk.call('textUndoer::reset', self.undoerID)
