#!/usr/bin/env python
import os, sys, string, traceback, regsub, Tkinter
import DialogManager, CommandPacket
import ptBuffer, ptInputText, ptInterpTrace, ptMegaWidgets, ptObjectViewer

import ptModular

_home_dir = os.path.dirname(__file__)

def _print(file, str='', terminator='\n'):
    file.write(str+terminator)

class Interpreter(Tkinter.Frame):

    Name = 'PTUI'
    Description = 'Python/Tkinter User Interface'
    Version = '1.0'
    Author = 'Zachary Roadhouse'

    AboutString = '%s (%s) v%s written by %s' % (Name, Description,
						 Version, Author)

    def __init__(self,root,namespace={},logfiles=1):
	Tkinter.Frame.__init__(self, root)
	self.pack(expand='y',fill='both',padx=2)

	self.root = root

	self.namespace = namespace
	self._initNamespace()

	self.dir = os.curdir
	self.tempDir = os.curdir
	self.logfiles = logfiles

	self._initTkWidgets()

	self.bufManager = ptBuffer.InterpBufferManager(self)

    def _initNamespace(self):
	self.namespace['Tkinter'] = Tkinter
	self.namespace['__builtins__'] = __builtins__

	if not self.namespace.has_key('self'):
	    self.namespace['self'] = self

    def _initTkWidgets(self):
	self._createMenubar()
	self._createToolBox()
	self._createTextAreas()
	self._createMessageArea()
	self._createBindings()

	self.stdout = self.oText
	self.stderr = self.eText

    def _createMenubar(self):
	self.menuBar = Tkinter.Frame(self,relief='raised',bd=2)
	self.menuBar.pack(side='top',fill='x')


	self.fileMenu = Tkinter.Menubutton(self.menuBar,
					   text='File')
	self.fileMenu.pack(side='left',padx=5)
	self.fileMenu.menu = Tkinter.Menu(self.fileMenu)
	self.fileMenu['menu'] = self.fileMenu.menu


	self.fileMenu.menu.add_command(label='Open...',
				       command=self.openScript,
				       accelerator='m-o')
	self.fileMenu.menu.add_command(label='Open Multiple...',
				       command=self.openMultipleScripts)
	self.fileMenu.menu.add_command(label='Append...',
				       command=self.appendScript)
	self.fileMenu.menu.add_separator()


	self.fileMenu.menu.add_command(label='Save',
				       command=self.saveBuffer,
				       accel='m-s')
	self.fileMenu.menu.add_command(label='Save as...',
				       command=self.saveBufferAs,
				       accel='m-Sh-s')
	self.fileMenu.menu.add_separator()


	self.fileMenu.menu.add_command(label='Delete Buffer',
				       command=self.deleteBuffer)
	self.fileMenu.menu.add_separator()


	self.fileMenu.menu.add_command(label='Clear Output',
				       command=self.clearOutput)
	self.fileMenu.menu.add_command(label='Clear Error',
				       command=self.clearError)
	self.fileMenu.menu.add_separator()


	self.fileMenu.menu.add_command(label='Close',
				       command=self.cmdClose)


	self.editMenu = Tkinter.Menubutton(self.menuBar,
					   text='Edit')
	self.editMenu.pack(side='left',padx=5)
	self.editMenu.menu = Tkinter.Menu(self.editMenu)
	self.editMenu['menu'] = self.editMenu.menu


	self.editMenu.menu.add_command(label='Undo',
				       command=self.undo)
	self.editMenu.menu.add_separator()


	self.editMenu.menu.add_command(label='Cut',
				       accel='C-w',
				       command=self.cut)
	self.editMenu.menu.add_command(label='Copy',
				       accel='M-w',
				       command=self.copy)
	self.editMenu.menu.add_command(label='Paste',
				       accel='C-y',
				       command=self.paste)
	self.editMenu.menu.add_separator()


	self.editMenu.menu.add_command(label='Goto line...',
				       accel='m-g',
				       command=self.gotoLine)
	self.editMenu.menu.add_command(label='Find...',
				       accel='A-f',
				       command=self.findExpression)

	self.bufferMenu = Tkinter.Menubutton(self.menuBar,
					     text='Buffers')
	self.bufferMenu.pack(side='left',padx=5)
	self.bufferMenu.menu = Tkinter.Menu(self.bufferMenu)
	self.bufferMenu['menu'] = self.bufferMenu.menu


	self.scriptMenu = Tkinter.Menubutton(self.menuBar,
					     text='Script')
	self.scriptMenu.pack(side='left',padx=5)
	self.scriptMenu.menu = Tkinter.Menu(self.scriptMenu)
	self.scriptMenu['menu'] = self.scriptMenu.menu


	self.scriptMenu.menu.add_command(label='Go!',
					 command=self.execute,
					 accel='C-g')
	self.scriptMenu.menu.add_command(label='Modularize!',
					 command=self.modularize)					 
	self.scriptMenu.menu.add_separator()


	self.scriptMenu.menu.add_command(label='Comment Region',
					 command=self.commentRegion,
					 accel='C-c')
	self.scriptMenu.menu.add_command(label='Uncomment Region',
					 command=self.uncommentRegion,
					 accel='M-c')
	self.scriptMenu.menu.add_separator()


	self.scriptMenu.menu.add_command(label='Indent Region',
					 command=self.indentRegion,
					 accel='C->')
	self.scriptMenu.menu.add_command(label='Dedent Region',
					 command=self.dedentRegion,
					 accel='C-<')
	self.scriptMenu.menu.add_separator()


	self.scriptMenu.menu.add_command(label='Examine variable',
					 command=self.examineVariable,
					 accel='M-e')

	self.scriptMenu.menu.add_command(label='Browse variables',
					 command=self.browseVariables,
					 accel='A-b')
	

	self.helpMenu = Tkinter.Menubutton(self.menuBar,
					   text='Help')
	self.helpMenu.pack(side='right',padx=5)
	self.helpMenu.menu = Tkinter.Menu(self.helpMenu)
	self.helpMenu['menu'] = self.helpMenu.menu


	self.helpMenu.menu.add_command(label='About PTUI...',
				       command=self.about)
	self.helpMenu.menu.add_separator()


	self.helpMenu.menu.add_command(label='Keyboard bindings...',
				       command=self.help_bindings)

	self.helpMenu.menu.add_command(label='Tutorial...',
				       command=self.help_tutorial)

    def _createTextAreas(self):
    	frame = Tkinter.Frame(self)
    	frame.pack(side='top',fill='x')
    	
	self.iLabel = Tkinter.Label(frame,text='Input')
	self.iLabel.pack(side='left')
	
	self.filenameLabel = Tkinter.Label(frame, text='No Buffer')
	self.filenameLabel.pack(side='right')


	inputFont = '-*-courier-medium-r-*-*-*-120-*-*-*-*-iso8859-*'
	self.iText = ptInputText.Text(self)
	self.iText.text.config(height=20,font=inputFont,bg='white')

	self.oLabel = Tkinter.Label(self,text='Output')
	self.oLabel.pack(side='top',anchor='w')

	self.oText = ptMegaWidgets.ScrolledText(self,readonly=1)
	self.oText.text.config(height=5,bg='white')

	self.eLabel = Tkinter.Label(self,text='Error')
	self.eLabel.pack(side='top',anchor='w')

	self.eText = ptMegaWidgets.ScrolledText(self,readonly=1)
	self.eText.text.config(height=5,bg='white')

    def _createToolBox(self):
    	frame = Tkinter.Frame(self,relief='groove',bd=2)
    	frame.pack(side='top',fill='x',padx=5,pady=5)
    	

    	sub = Tkinter.Frame(frame)
	sub.pack(side='left',padx=5,pady=5)

	self.goButton = Tkinter.Button(sub, text='Go!',
				       command=self.execute)
	self.goButton.pack(side='left')

	self.modButton = Tkinter.Button(sub,text='Modularize!',
					command=self.modularize)
	self.modButton.pack(side='left')


    	sub = Tkinter.Frame(frame)
	sub.pack(side='left',padx=5,pady=5)

	self.commentButton = Tkinter.Button(sub, text='Comment',
					    command=self.commentRegion)
	self.commentButton.pack(side='left')

	self.uncommentButton = Tkinter.Button(sub, text='Uncomment',
					      command=self.uncommentRegion)
	self.uncommentButton.pack(side='left')


	sub = Tkinter.Frame(frame)
	sub.pack(side='left',padx=5,pady=5)

	self.indentButton = Tkinter.Button(sub, text='Indent',
					    command=self.indentRegion)
	self.indentButton.pack(side='left')

	self.dedentButton = Tkinter.Button(sub, text='Dedent',
					      command=self.dedentRegion)
	self.dedentButton.pack(side='left')


	sub = Tkinter.Frame(frame)
	sub.pack(side='left',padx=5,pady=5)

	self.examineButton = Tkinter.Button(sub,text='Examine',
					    command=self.examineVariable)
	self.examineButton.pack(side='left')


	self.toolBox = frame

    def _createMessageArea(self):
	self.messageArea = Tkinter.Label(self,text='Ready',relief='sunken',bd=1)
	self.messageArea.pack(side='top',anchor='w',padx=5,pady=5,fill='x')

    def examineVariable(self,event=None):
	sel = self.iText.text.tag_ranges('sel')
	if sel:
	    var_path = string.splitfields(self.iText.text.get(sel[0],
							      sel[1]),'.')
	    var_name = var_path[0]
	    del var_path[0]

	    if self.bufManager.current.namespace.has_key(var_name):
		obj = self.bufManager.current.namespace[var_name]

		name = var_name
		for attr in var_path:
		    name = name + '.' + attr
		    if hasattr(obj,attr):
			obj = getattr(obj,attr)
		    else:
			msg = 'No object '+name+' exists in this buffer.'
			self.showMessage(msg,3000)
			return

		okObj = hasattr(obj,'__dict__') or \
			hasattr(obj,'__members__') or \
			(type(obj) == type({}))

		if not okObj:
		    obj = {name:obj}

		bw = ptObjectViewer.makeDefaultViewer(self,obj)
		bw.title(name)
	    else:
		message = 'No object '+var_name+' exists in this buffer.'
		self.showMessage(message,3000)
	else:
	    self.showMessage('No selection to examine.',3000)

    def browseVariables(self,event=None):
	bw = ptObjectViewer.makeDefaultViewer(self,
					    self.bufManager.current.namespace)
	bw.title(self.bufManager.current.name + "'s Variables")

    # This method is not thread safe because of the swapping
    # of standard output and error during execution
    def execute(self,event=None):
	sel = self.iText.text.tag_ranges('sel')

	data = None
	if sel:
	    data = self.iText.text.get(sel[0],sel[1])
	else:
	    data = self.iText.text.get('1.0','end')

	if len(data) <= 1: return

	self.swapStdFiles()
	self.showMessage('Executing...')
	self.update()

	code = None

	try:
	    code = compile(data,'<PTUI>','exec')
	except SyntaxError,value:
	    self.hiliteError(self.handleSyntaxError(value))

	if code:
	    try:
		exec(code,self.bufManager.current.namespace)
	    except:
		lineno = ptInterpTrace.printTraceBack(sys.exc_traceback)	    
		self.hiliteError(lineno)

	self.swapStdFiles()     
	self.showMessage('Ready')

    def swapStdFiles(self):
	sys.stdout,self.stdout = self.stdout,sys.stdout
	sys.stderr,self.stderr = self.stderr,sys.stderr

    def handleSyntaxError(self,value):
	lines = traceback.format_exception_only(SyntaxError,value)

	head = string.splitfields(lines[0],',')
	head[0] = "In the current buffer"
	
	lines[0] = string.joinfields(head,',')

	sys.stderr.write(string.join(lines))

	return value[1][1]

    def hiliteError(self,lineno):
	sel = self.iText.text.tag_ranges('sel')
	if sel:
	    args = ('sel',) + sel
	    apply(self.iText.text.tag_remove,args)
	    fromline = string.splitfields(sel[0],'.')[0]
	    lineno = string.atoi(fromline) + lineno - 1

	lindex = '%d.0' % lineno

	self.iText.text.tag_add('sel',lindex,lindex + ' lineend')
	self.iText.text.yview(lindex)

    def commentRegion(self,event=None):
	if not self.iText.commentRegion(event):
	    self.showMessage('No selection to comment.',3000)

    def uncommentRegion(self,event=None):
	if not self.iText.uncommentRegion(event):
	    self.showMessage('No selection to uncomment.',3000)

    def indentRegion(self,event=None):
	if not self.iText.indentRegion(event):
	    self.showMessage('No selection to ident.',3000)

    def dedentRegion(self,event=None):
	if not self.iText.dedentRegion(event):
	    self.showMessage('No selection to dedent.',3000)

    def saveBufferAs(self,event=None):
	fd = DialogManager.SaveFile(self)
	filename = fd.go(self.dir,'*.py')

	if filename:
	    self.dir = os.path.dirname(filename)

	    self.bufManager.saveBufferAs(filename)

	    self.showMessage('Saved buffer as %s' % filename,3000)

    def saveBuffer(self,event=None):
	self.bufManager.saveBuffer()

	self.showMessage('Saved buffer %s' % self.filename,3000)

    def openScript(self,event=None):
	d = DialogManager.OpenFile(self)
	filename = d.go(self.dir,'*.py') 

	if filename:
	    self.dir = os.path.dirname(filename)

	    if os.path.isfile(filename):
		self.bufManager.addBuffer(filename)
	    else:
		self.showMessage('Bad filename: %s' % filename,3000)

    def openMultipleScripts(self,event=None):
	d = DialogManager.OpenMultipleFiles(self)

	filelist = d.go(self.dir,'*.py') 
	if filelist:
	    self.dir = os.path.dirname(filelist[0])

	    for filename in filelist:
		if os.path.isfile(filename):
		    buf = self.bufManager.newBuffer(filename)
		    try:
			buf.updateFromFile()
			self.bufManager.buffers[buf.name] = buf
		    except:
			self.showMessage('Error in loading file: %s' 
					 % filename,3000)
		else:
		    self.showMessage('Bad filename: %s' % filename,3000)

	    buf.switchTo()

    def appendScript(self,event=None):
	d = DialogManager.OpenFile(self)
	filename = d.go(self.dir,'*.py') 
	if filename:
	    self.appendFile(filename)

    def appendFile(self,filename):
	head, tail = os.path.split(filename)
	if head: self.dir = head

	input = open(filename,'r')
	data = input.read()
	input.close()
	self.iText.text.insert('end',data)

    def deleteBuffer(self,event=None):
	self.bufManager.deleteBuffer()

    def clearOutput(self,event=None):
	self.oText.text.config(state='normal')
	self.oText.text.delete('1.0','end')
	self.oText.text.config(state='disabled')

    def clearError(self,event=None):
	self.eText.text.config(state='normal')
	self.eText.text.delete('1.0','end')
	self.eText.text.config(state='disabled')

    def cleanup(self):
	if self.logfiles:
	    data = self.oText.text.get('1.0','end')
	    log = open(os.path.join(self.tempDir,'Output.log'),'a')
	    log.write(data)
	    log.close()

	    data = self.eText.text.get('1.0','end')
	    log = open(os.path.join(self.tempDir,'Error.log'),'a')
	    log.write(data)
	    log.close()

    def destroy(self):
	self.cleanup()
	Tkinter.Frame.destroy(self)

    def cmdClose(self,event=None):
	self.cleanup()
	self.tk.quit()

    def _createBindings(self):
	self.root.bind('<Meta-o>',self.openScript)
	self.root.bind('<Meta-s>',self.saveBuffer)
	self.root.bind('<Meta-S>',self.saveBufferAs)

	try:
	    self.root.bind('<<Cut>>',self.cut)
	    self.root.bind('<<Copy>>',self.copy)

	except Tkinter.TclError:
	    self.root.bind('<Control-w>',self.cut)
	    self.root.bind('<Meta-w>',self.copy)

	self.root.bind('<Meta-g>',self.gotoLine)
	self.root.bind('<Alt-f>',self.findExpression)

	self.root.bind('<Control-g>',self.execute)

	self.root.bind('<Control-c>',self.commentRegion)
	self.root.bind('<Meta-c>',self.uncommentRegion)

	self.root.bind('<Control-greater>',self.indentRegion)
	self.root.bind('<Control-less>',self.dedentRegion)

	self.root.bind('<Alt-b>',self.browseVariables)
	self.root.bind('<Meta-e>',self.examineVariable)

    def setFilename(self,name):
	self.filename = name
	self.filenameLabel.config(text=os.path.split(name)[1])

	if name == 'Untitled':
	    self.fileMenu.menu.entryconfig('Save',state='disabled')
	    self.root.unbind('<Meta-s>')
	else:
	    self.fileMenu.menu.entryconfig('Save',state='normal')
	    self.root.bind('<Meta-s>',self.saveBuffer)
	    
    def help_bindings(self,event=None):
	help_path = os.path.join(_home_dir,'Help-Bindings.txt')

	hf = open(help_path)
	data = hf.read()
	hf.close()

	data = regsub.sub('@@NAME@@',Interpreter.Name,data)
	data = regsub.sub('@@VERSION@@',Interpreter.Version,data)

	hw = ptMegaWidgets.HelpWindow(self,data)
	hw.title(Interpreter.Name + ' Keyboard Bindings')
	hw.center(self.root)

    def help_tutorial(self,event=None):
	help_path = os.path.join(_home_dir,'Help-Tutorial.txt')

	hf = open(help_path)
	data = hf.read()
	hf.close()

	data = regsub.sub('@@NAME@@',Interpreter.Name,data)
	data = regsub.sub('@@VERSION@@',Interpreter.Version,data)

	hw = ptMegaWidgets.HelpWindow(self,data)
	hw.title(Interpreter.Name + ' Tutorial')
	hw.center(self.root)

    def about(self,event=None):
	cmd = CommandPacket.CommandPacket(name='Ok',function='close',args=None)
	aw = ptMegaWidgets.AlertDialog(self.master,
				     'About Interpreter',
				     self.AboutString,
				     'questhead',
				     cmd)

    def gotoLine(self,event=None):
	gw = ptMegaWidgets.TextGotoDialog(self.master,self.iText.text)

    def findExpression(self,event=None):
	fw = ptMegaWidgets.TextFindDialog(self.master,self.iText.text)

    def showMessage(self,message,life_time=None):
	self.messageArea.config(text=message)
	self.messageArea.update()

	if life_time:
	    cmd = CommandPacket.CommandPacket(self.showMessage,('Ready',))
	    self.messageArea.after(life_time,cmd.doit)

    def undo(self,*event):
	self.iText.undo(event)

    def cut(self,event=None):
	wid = apply(self.tk.call,
		    ('selection','own','-displayof',self._w))
	
	if wid:
	    try:
		sel = self.selection_get()
		self.clipboard_clear()
		self.clipboard_append(sel)
		self._nametowidget(wid).delete('sel.first','sel.last')
	    except Tkinter.TclError:
		pass
		
    def copy(self,event=None):

	try:
	    sel = self.selection_get()
	    self.clipboard_clear()
	    self.clipboard_append(sel)
	except Tkinter.TclError:
	    pass

    def paste(self,event=None):
	try:
	    sel = self.selection_get(selection='CLIPBOARD')
	    self.iText.text.insert('insert',sel)
	except Tkinter.TclError:
	    pass

    def modularize(self,event=None):
    	ptModular.import_buffer(self.bufManager.current)
