import regsub, string, sys
import Tkinter,CommandPacket

class LabeledEntry(Tkinter.Frame):
    def __init__(self,root,text,var,initial,width=10):
	Tkinter.Frame.__init__(self,root)
	self.pack(side=Tkinter.TOP,fill=Tkinter.X)
	
	self.var = var
	self.var.set(initial)
	
	self.label = Tkinter.Label(self,text=text)
	self.label.pack(side=Tkinter.LEFT)
	
	self.entry = Tkinter.Entry(self,
				   width = width,
				   textvariable=self.var)
	self.entry.pack(side=Tkinter.RIGHT)
	
    def get(self):
	return self.var.get()
	
    def set(self,value):
	return self.var.set(value)

class ScrolledText(Tkinter.Frame):
     def __init__(self,root,text=None,readonly=None):
	  Tkinter.Frame.__init__(self,root)
	  self.pack(side=Tkinter.TOP,
		    expand=Tkinter.YES,
		    fill=Tkinter.BOTH)

	  self._initTkWidgets()

	  if text:
	       self.text.insert(Tkinter.END,text)

	  if readonly:
	       self.text.config(state=Tkinter.DISABLED)

     def _initTkWidgets(self):
	  self.scroll = Tkinter.Scrollbar(self)
	  self.scroll.pack(side=Tkinter.RIGHT,
			   fill=Tkinter.Y)

	  self.text = Tkinter.Text(self,
				   wrap=Tkinter.WORD,
				   yscrollcommand=self.scroll.set)
	  self.text.pack(side=Tkinter.RIGHT,
			 expand=Tkinter.Y,
			 fill=Tkinter.BOTH)

	  self.scroll.config(command=self.text.yview)
	  
     def write(self,strng):
	  if strng is None:
	       raise TypeError, 'string, ' + `strng`

	  flag = (self.text.config('state')[-1] == 'disabled')

	  if flag: self.text.config(state='normal')

	  self.text.insert('end',strng)
	  self.text.yview('end')

	  if flag: self.text.config(state='disabled')

     def writelines(self,sList):
	  for strng in sList:
	       self.write(strng)

class BasicWindow(Tkinter.Toplevel):
    def __init__(self, master, cnf={}, **kw):
	apply(Tkinter.Toplevel.__init__,(self,master,cnf),kw)
	self._initTkWidgets()

    def _initTkWidgets(self):
	pass

    def center(self,bottom,relx=0.5,rely=0.5):
	"""move the top window to be over the bottom, centers aligned."""

	if bottom.winfo_ismapped():
	    b_width = bottom.winfo_width()
	    b_height = bottom.winfo_height()
	    b_x = bottom.winfo_rootx()
	    b_y = bottom.winfo_rooty()
	else:
	    b_width = bottom.winfo_screenwidth()
	    b_height = bottom.winfo_screenheight()
	    b_x = b_y = 0

	width = self.winfo_reqwidth()
	height = self.winfo_reqheight()

	x = b_x + (b_width - width) * relx
	y = b_y + (b_height - height) * rely

	self.geometry('+%d+%d' % (x,y))

class HelpWindow(BasicWindow):
    def __init__(self,root,helptext):
	self.helptext = helptext

	BasicWindow.__init__(self,root)
	self.withdraw()
	self.update_idletasks()
	self.center(root)
	self.deiconify()
    
    def _initTkWidgets(self):
	self.hText = ScrolledText(self,self.helptext,readonly=1)

	sub = Tkinter.Frame(self)
	sub.pack(side='top',fill='x')

	self.ok = Tkinter.Button(sub,
				 text='Ok',
				 command = self.destroy)
	self.ok.pack(padx=5,pady=5,side='right')

	self.find = Tkinter.Button(sub,
				   text='Find',
				   command = self.cmdFind)
	self.find.pack(padx=5,pady=5,side='left')
	
    def cmdFind(self):
	fw = TextFindDialog(self,self.hText.text)

class DialogWindow(BasicWindow):
    def __init__(self,master,title,text,bitmap,default,commands,
		 transient=None,centered=None):
	self.text = text
	self.bitmap = bitmap
	self.default = default
	self.commands = commands

	BasicWindow.__init__(self,master,class_='Dialog')

	self.withdraw()

	self.title(title)
	self.iconname('Dialog')

	parent = master.winfo_toplevel()
	if transient: 
	    self.transient(parent)

	if centered:
	    self.update_idletasks()
	    self.center(parent)
	
	self.deiconify()

    def _initTkWidgets(self):
	self.top = Tkinter.Frame(self,relief='raised',bd=1)
	self.top.pack(side='top',fill='both')

	self.bot = Tkinter.Frame(self,relief='raised',bd=1)
	self.bot.pack(side='bottom',fill='both')

	self.top.msg = Tkinter.Message(self.top,
				       width='3i',
				       text=self.text,
				       font='-*-Times-Medium-R-Normal-*-180-*')
	self.top.msg.pack(side='right',expand=1,fill='both',padx='3m',pady='3m')

	if self.bitmap:
	    self.top.bitmap = Tkinter.Label(self.top,bitmap=self.bitmap)
	    self.top.bitmap.pack(side='left',padx='3m',pady='3m')

	self.bot.buttons = []

	for i in range(len(self.commands)):
	    place = self.bot
	    if self.default == i:
		place = Tkinter.Frame(self.bot,relief='sunken',bd=1)
		place.pack(side='left',expand=1,pady='2m')

	    command = None
	    if self.commands[i].function == 'close':
		command = self.destroy
	    elif hasattr(self.commands[i],'doit'):
		command = self.commands[i].doit

	    name = self.commands[i].name or '<None>'

	    button = Tkinter.Button(place,
				    text=name,
				    command=command)

	    if self.default == i:
		button.bind('<Return>',button.flash)
		button.pack(padx='2m',
			    pady='2m',
			    ipadx='2m',
			    ipady='1m',
			    side='left')
	    else:
		button.pack(padx='3m',
			    pady='3m',
			    ipadx='2m',
			    ipady='1m',
			    side='left',
			    expand=1)

	    self.bot.buttons.append(button)

class AlertDialog(DialogWindow):

    """A transient dialog that will pop up at the center of the
    calling window."""

    def __init__(self,master,title,text,bitmap,command,centered=1):
	DialogWindow.__init__(self,master,title,text,
			      bitmap,0,[command],
			      transient=1,centered=centered)

class TextGotoDialog(DialogWindow):
    def __init__(self,master,textWidget):
	cmdCancel = CommandPacket.CommandPacket(name='Cancel',
						  function='close',
						  args=None)

	cmdGoto = CommandPacket.CommandPacket(name='Goto',
					      function=self.gotoLine,
					      args=None)

	DialogWindow.__init__(self,master,
			      'Goto Line','','questhead',
			      0,[cmdGoto,cmdCancel],centered=1)

	self.textWidget = textWidget

    def _initTkWidgets(self):
	DialogWindow._initTkWidgets(self)

	self.top.goto = LabeledEntry(self.top,'Goto Line',Tkinter.IntVar(),1)
	self.top.goto.pack(side='right',padx=5,pady=5)

	self.bind('<Key-Return>',self.gotoLine)

    def center(self,bottom,relx=0.5,rely=0.8):
	DialogWindow.center(self,bottom,relx,rely)

    def gotoLine(self,event=None):
	pos = str(self.top.goto.get())+'.0'
	self.textWidget.yview(pos)
	self.textWidget.mark_set('insert',pos)
	self.destroy()

class TextFindDialog(DialogWindow):
    def __init__(self,master,textWidget):
	find_cmd = CommandPacket.CommandPacket(name='Find',
					       function=self.findExpression,
					       args=None)

	close_cmd = CommandPacket.CommandPacket(name='Cancel',
						function='close',
						args=None)

	DialogWindow.__init__(self,master,
			      'Find','','questhead',
			      0,[find_cmd,close_cmd],centered=1)

	self.textWidget = textWidget

    def _initTkWidgets(self):
	DialogWindow._initTkWidgets(self)

	self.top.find = LabeledEntry(self.top,'Find:',Tkinter.StringVar(),'',30)
	self.top.find.pack(side='right',padx=5,pady=5)
	
	self.mid = Tkinter.Frame(self,relief='raised',bd=1)
	self.mid.pack(after=self.top,side='top',fill='x')

	self.regexp_v = Tkinter.BooleanVar()
	self.case_v = Tkinter.BooleanVar()
	self.backwards_v = Tkinter.BooleanVar()

	self.regexp_checkbutton = Tkinter.Checkbutton(self.mid,
						      text="regexp",
						      variable=self.regexp_v)
##	self.regexp_checkbutton.pack(side='left')

	self.case_checkbutton = Tkinter.Checkbutton(self.mid,
						    text="case sensitive",
						    variable=self.case_v)
	self.case_checkbutton.pack(side='left')

	self.backwards_checkbutton = Tkinter.Checkbutton(self.mid,
							 text="backwards",
							 variable=self.backwards_v)
	self.backwards_checkbutton.pack(side='right',expand=1)

	self.bind('<Key-Return>',self.findExpression)

    def center(self,bottom,relx=0.5,rely=0.8):
	DialogWindow.center(self,bottom,relx,rely)
	  
    def updateView(self,pos,length):
	sel = self.textWidget.tag_ranges('sel')
	if sel:
	    args = ('sel',) + sel
	    apply(self.textWidget.tag_remove,args)

	lineno,charno = tuple(string.splitfields(pos,'.'))
	endpos = lineno + '.' + str(string.atoi(charno) + length)

	self.textWidget.yview(pos)
	self.textWidget.tag_add('sel',pos,endpos)

	if self.backwards_v.get():
	    endpos = lineno + '.' + str(string.atoi(charno) - length)

	self.textWidget.mark_set('insert',endpos)

    def findExpression(self,event=None):
	expression = self.top.find.get()

	if not expression:
	    self.destroy()
	    return

	start = self.textWidget.index('insert')

	pos = self.textWidget.search(expression,start,
				     backwards=self.backwards_v.get(),
				     regexp=self.regexp_v.get(),
				     nocase=not self.case_v.get())

	if pos:
	    self.updateView(pos,len(expression))
	else:
	    cmd = CommandPacket.CommandPacket(name='Ok',
					      function='close',
					      args=None)

	    dw = AlertDialog(self,
			     'Error',
			     'Could not find: ' + expression,
			     'error',cmd)


class InputDialog(DialogWindow):
    def __init__(self,master,question):
	self.question = question
	DialogWindow.__init__(self,master,
			      'Input','','questhead',
			      0,[],transient=1,centered=1)

    def _initTkWidgets(self):
	DialogWindow._initTkWidgets(self)

	self.top.input = LabeledEntry.LabeledEntry(self.top,self.question,
						   Tkinter.StringVar(),'',30)
	self.top.input.pack(side='right',padx=5,pady=5)

	self.bind('<Key-Return>',self.getResponse)

    def getResponse(self,event=None):
	self.quit(self.top.input.get())

    def input(self):
	self.top.input.entry.focus_set()
	self.grab_set()

	self.response = None
	self.master.mainloop() # Exited by self.quit(response)
	self.destroy()
	return self.response

    def quit(self, response = None):
	self.response = response
	self.master.quit()   # Exit mainloop()

class RawInputCaller:
    def __init__(self,master):
	self.master = master
    
    def __call__(self,prompt=''):
	idw = InputDialog(self.master,prompt)
	return idw.input()

class InputCaller:
    BadNamespace = "The statement sys.exc_traceback.tb_frame.f_back.f_(l/g)" \
		"is no longer accurate.  Please notify the author."

    def __init__(self,master):
	self.master = master
    
    def __call__(self,prompt=''):
	idw = InputDialog(self.master,prompt)

	hack = 'Hack for getting locals'
	try:
	    raise hack
	except hack:
	    locals = sys.exc_traceback.tb_frame.f_back.f_locals
	    globals = sys.exc_traceback.tb_frame.f_back.f_globals

	try:
	    locals
	    globals
	except:
	    print self.BadNamespace
	    return

	return eval(idw.input(),locals,globals)
	
	
if __name__ == "__main__":
    print 'Hello from ptMegaWidgets'
