import Tkinter,string,types
import ptMegaWidgets

_Sequences = (types.ListType,types.TupleType)
_Maps = (types.DictType,)


def listattrs(x,get_class=1,get_bases=1):

    """Returns a list of attributes that can be accessed using getattr.

    Takes an object as an arguement."""

    dictkeys = []
    if hasattr(x,'__dict__'):
	dictkeys = x.__dict__.keys()

    members = []
    if hasattr(x,'__members__'):
	members = x.__members__

    methods = []
    if hasattr(x,'__methods__'):
	methods = x.__methods__

    total = dictkeys + methods + members

    if get_class and hasattr(x,'__class__'):
	total = total + listattrs(x.__class__,get_class,get_bases)

    if get_bases and hasattr(x,'__bases__'):
	# It's a derived class; add the base class attributes
	for base in x.__bases__:
	    total = total + listattrs(base,get_class,get_bases)

    total.sort()

    # Remove duplicate entries
    i = 0
    while i+1 < len(total):
	if total[i] == total[i+1]:
	    del total[i+1]
	else:
	    i = i+1

    return total


class Attribute(Tkinter.Frame):

    BadValue = 'Bad value for Attribute object.'
    NoValueWidget = 'Your must override _initValueWidget.'

    def __init__(self,master,name,value):
	self.name = name
	self.value = self._initValue(value)

	Tkinter.Frame.__init__(self,master)
	self.pack(side='top',fill='x')
		
	self._initTkWidgets()

    def _initValue(self,value):
	if self.checkValue(value):
	    return value

	raise Attribute.BadValue

    def checkValue(self,value):
	return 1

    def _initTkWidgets(self):
	self._initValueWidget()
	
	label = Tkinter.Label(self,text=self.name)
	label.pack(side='right')

    def _initValueWidget(self):
	raise Attribute.NoValueWidget

    def viewObject(self):
	viewer = viewerExists(self.value)

	if viewer:
	    viewer.master.lift()
			
	else:
	    self.makeViewer(self.winfo_toplevel(),self.value)

    def makeViewer(self,master,object):
	return makeDefaultViewer(master,object)

class GeneralAttribute(Attribute):
    def _initValue(self,value):
	val = Tkinter.StringVar()
	val.set(`value`)

	return val
	    
    def _initValueWidget(self):
	entry = Tkinter.Entry(self,textvariable=self.value)
	entry.pack(side='left')

class ObjectAttribute(Attribute):
    def checkValue(self,value):
	return hasattr(value,'__dict__') or \
	       hasattr(value,'__members__') or \
	       hasattr(value,'__methods__')

    def _initButtonText(self):
	return '%s' % string.split(`self.value`[1:-1])[0]

    def _initValueWidget(self):
	button = Tkinter.Button(self,
				text=self._initButtonText(),
				command=self.viewObject,
				width=18)
	button.pack(side='left')

    def makeViewer(self,master,object):
	bw = ptMegaWidgets.BasicWindow(master)
	bw.title(`object`[1:-1] + "'s Variables")
	bw.attrViewer = ObjectViewer(bw,object)

	return bw

class DocAttribute(ObjectAttribute):
    def checkValue(self,value):
	return self.name == '__doc__'

    def _initButtonText(self):
	if self.value == None:
	    return 'No docs'
	
	if len(self.value) > 10:
	    return self.value[:10] + '...'

	return text

    def makeViewer(self,master,object):
	if object:
	    return ptMegaWidgets.HelpWindow(master,object)

class DictionaryAttribute(ObjectAttribute):
    def _initButtonText(self):
	return 'dict'

    def checkValue(self,value):
	return type(value) == types.DictType

    def makeViewer(self,master,object):
	Attribute.makeViewer(self,master,object)

class ListAttribute(ObjectAttribute):
    def _initButtonText(self):
	return 'list'

    def checkValue(self,value):
	return type(value) == types.ListType

    def makeViewer(self,master,object):
	bw = ptMegaWidgets.BasicWindow(master)
	bw.title("List at %s" % hex(id(object)))
	bw.attrViewer = SequenceViewer(bw,object)

	return bw

class TupleAttribute(ObjectAttribute):
    def _initButtonText(self):
	return 'tuple'

    def checkValue(self,value):
	return type(value) == types.TupleType

    def makeViewer(self,master,object):
	bw = ptMegaWidgets.BasicWindow(master)
	bw.title("Tuple at %s" % hex(id(object)))
	bw.attrViewer = SequenceViewer(bw,object)

	return bw

class AttributeViewer(Tkinter.Frame):

    Viewers = []

    NoAttributeType = 'No Attribute type to deal with value.'

    def __init__(self, master, object):
	AttributeViewer.Viewers.append(self)

	Tkinter.Frame.__init__(self,master)
	self.pack(side='top',fill='both',padx=5,pady=5,expand=1)

	self.object = object
	self.attributes = []
	self.attr_ids = []

	self._initTkWidgets()

    def _initTkWidgets(self):
	## Create the top of the window
	sub = Tkinter.Frame(self)
	sub.pack(side='top',fill='x')

	label = Tkinter.Label(sub,
			      text=self._initObjectText())
	label.pack(side='left')

	closeB = Tkinter.Button(sub,
				text='Close',
				command=self.master.destroy)
	closeB.pack(side='right')
	
	sub = Tkinter.Frame(self)
	sub.pack(side='top',fill='x')

	label = Tkinter.Label(sub,text='Show:')
	label.pack(side='left')

	self.showClass_p = Tkinter.BooleanVar()
	self.showInherited_p = Tkinter.BooleanVar()

	showClass = Tkinter.Checkbutton(sub,
					text='Class members',
					variable=self.showClass_p)
	showClass.pack(side='left')

	showInherited = Tkinter.Checkbutton(sub,
					    text='Inherited members',
					    variable=self.showInherited_p)
	showInherited.pack(side='left')

	self.showClass_p.set(0)
	self.showInherited_p.set(0)

	## Create the scrollbar and canvas
	self.attrBar = Tkinter.Scrollbar(self)
	self.attrBar.pack(side='right',fill='y')

	sub = Tkinter.Frame(self,relief='sunken',bd=2)
	sub.pack(side='right',fill='both',expand=1)
	
	self.attrBox = Tkinter.Canvas(sub,
				      width = 400,
				      height = 200,
				      yscrollcommand=self.attrBar.set,
				      yscrollincrement=40)
	self.attrBox.pack(side='top',fill='both')

	self.attrBar.config(command=self.attrBox.yview)

	self._initAttributes()

	showInherited.config(command=self.updateView)
	showClass.config(command=self.updateView)

    def _initObjectText(self):
	objID = hex(id(self.object))
	if type(self.object) == types.DictType:
	    return "Dict at %s" % objID

	if type(self.object) == types.ListType:
	    return "List at %s" % objID

	if type(self.object) == types.TupleType:
	    return "Tuple at %s" % objID

	return `self.object`[1:-1]

    def _initAttributes(self):
	attributes = self.getAttributeList(self.object)

	xpos = 10
	ypos = 40
	yskip = 40

	for name in attributes:
	    value = self.getAttribute(self.object,name)
	    
	    attr = self.chooseAttribute(name,value)

	    self.attr_ids.append(self.attrBox.create_window(xpos,ypos,
							    window=attr,
							    anchor='nw'))

	    self.attributes.append(attr)
	    ypos = ypos + yskip

	scrollRegion = '0 0 400 %i' % ypos
	self.attrBox.config(scrollregion=scrollRegion)

    def getAttributeList(self,object):
	if type(object) in _Maps:
	    keys = object.keys()
	    keys.sort
	    return keys
	
	if type(object) in _Sequences:
	    return range(len(object))

	return listattrs(object,
			 self.showClass_p.get(),
			 self.showInherited_p.get())

    def getAttribute(self,object,name):
	if type(object) in (_Maps + _Sequences):
	    return object[name]

	return getattr(object,name)

    def destroy(self):
	AttributeViewer.Viewers.remove(self)
	Tkinter.Frame.destroy(self)

    def chooseAttribute(self,name,value):
	attr_types = []

	## Simple types
	attr_types.append(DictionaryAttribute)
	attr_types.append(TupleAttribute)
	attr_types.append(ListAttribute)

	## Complex types
	attr_types.append(ObjectAttribute)

	## Special cases
	attr_types.append(DocAttribute)

	## Everything else
	attr_types.append(GeneralAttribute)
	
	attr = None
	for ty in attr_types:
	    try:
		attr = ty(self.attrBox,name,value)
		break
	    except Attribute.BadValue: pass

	if attr: return attr

	raise AttributeViewer.NoAttributeType

    def updateView(self):
	self.update_idletasks()

	canv_del = self.attrBox.delete
	objs = self.attr_ids
	map(canv_del,objs)

	self.attr_ids = []
	self.attributes = []

	self._initAttributes()

class ObjectViewer(AttributeViewer):
    pass

class SequenceViewer(AttributeViewer):
    pass

def viewerExists(obj):
    result = filter(lambda a,obj=obj: a.object == obj,
		    AttributeViewer.Viewers)
    if result: return result[0]

def makeDefaultViewer(master,object):
    bw = ptMegaWidgets.BasicWindow(master)

    if type(object) == types.DictType:
	bw.title("Dict at %s" % hex(id(object)))
    elif type(object) == types.ListType:
	bw.title("List at %s" % hex(id(object)))
    elif type(object) == types.TupleType:
	bw.title("Tuple at %s" % hex(id(object)))
    else:
	bw.title(`object`[1:-1] + "'s Variables")

    bw.attrViewer = AttributeViewer(bw,object)
     
    return bw
		
def test():
    app = Tkinter.Tk()

    class Blah:

	"""Does nothing."""

	def __init__(self):
	    """Does nothing as well."""
	    pass

    nsp = { 'Tkinter':Tkinter,
	    '__builtins__':__builtins__,
	    'app':app,
	    'AttributeViewer':AttributeViewer,
	    'a_blah':Blah() }

    av = AttributeViewer(app,nsp)
    
    app.mainloop()

if __name__ == '__main__':
    test()
